/*****************************************************
Project : Mastermodul
Version :
Date    : 16.06.2008
Author  : Joakim Hammer
Company : NTNU
Comments:


Chip type           : ATmega32L
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External RAM size   : 0
Data Stack size     : 512
*****************************************************/

#include<stdio.h>
#include<string.h>
#include "nodetable.h"

#define NODE1		0x0A
#define NODE2		0x0B
#define NODE3		0x0C
#define BROADCAST	0xFF
#define ANTALL_NODER	5
#define CONFIG_DELAY 10//us

int number_of_nodes;
char node_table[ANTALL_NODER][ANTALL_NODER];
char address_list[ANTALL_NODER];
int neigbours[ANTALL_NODER];

void init_nodetable(void){
	number_of_nodes =0;
	memset(node_table, 0x00, sizeof(node_table));

}
int registered_node(char node){
	int i;
		for(i=0;i<number_of_nodes; i++){
			if(address_list[i] == node)
				return i;
		}
		return -1;
}
void print_table(int number_of_nodes){
	int i, j;
		for(i=0;i<number_of_nodes; i++){
			for(j=0;j<number_of_nodes; j++){
				usart_printstring("%d -",node_table[i][j]);
			}
			usart_printstring("\n");
		}
}
int update_node_table(char *text, int msg_len)
{
		int adresse1 = 0;
		int adresse2 = 0;

		//sjekk om noden er registrert
		adresse1 = registered_node(text[0]);
		if(adresse1 <0 ){
			address_list[number_of_nodes] = text[0];
			adresse1 = number_of_nodes;
			number_of_nodes++;
		}
		//legg til den nye nodens avstand fra sentral
		if(node_table[adresse1][0] == 0 || node_table[adresse1][0] > msg_len )
			node_table[adresse1][0]= msg_len;

		int i=0;
		//oppdater resten av nodetabellen
		while(i < msg_len){
			adresse2 = registered_node(text[i]);
			if(adresse2 <0 ) adresse2 = 0;
			if(node_table[adresse1][adresse2+1] == 0 || node_table[adresse1][adresse2+1] > i ){
				node_table[adresse1][adresse2+1] = i;
			}
			i++;
		}

		print_table(number_of_nodes);
		usart_printstring("%s\n",address_list);

	return 1;
}

int remove_connection(char *text, int msg_len)
{
	int sender = registered_node(text[0]);
	int missing_node = registered_node(text[1]);

	if(sender <0 || missing_node <0)
		return -1;
	//else:
	node_table[sender][missing_node+1] = 0;
	node_table[missing_node][sender+1] = 0;
	return 1;
}

int find_neighbours(char my_addr){
	int i,j, addr;
	usart_printstring("%d noder,  ",number_of_nodes);

	memset(neigbours, 0xFF, sizeof(neigbours));

	addr = registered_node(my_addr);
	j = 0;

	//lete i både x og y retning
	for(i=0;i<number_of_nodes; i++){
		if (node_table[i][addr+1] == 1){
			neigbours[j] = i+1;
			j++;
		}
	}
	for(i=0;i<number_of_nodes; i++){
		if (node_table[addr][i] == 1){
			if(i==0) neigbours[j] = 0;
			else neigbours[j] = i;
			j++;
		}
	}

	return 1;
}
char comp_neighbours(char my_addr, char recipent){
	find_neighbours(my_addr);
//	usart_printstring("\nmy neighbours: %d %d %d %d\n",neigbours[0],neigbours[1],neigbours[2],neigbours[3]);

	int recipent_addr = registered_node(recipent)+1;

	int i;
		for(i=0;i<number_of_nodes; i++){
			if(neigbours[i] == recipent_addr ){
//				usart_printstring("fant match blandt naboer %d - %c\n", recipent_addr, recipent);
				return recipent_addr;
			}
		}
//	usart_printstring("fant ikke match blandt naboer: ");
	int shortpath = ANTALL_NODER;
	char best_choice = 0xFF;

	for(i=0;i<number_of_nodes; i++){
//		usart_printstring("res %d, nabo %d, innhold %d\n", recipent_addr, neigbours[i], node_table[recipent_addr][neigbours[i]]);

		if(neigbours[i] < 0)
			;
		//retning inn til sentralen
		else if (node_table[neigbours[i]-1][recipent_addr] > 0 && node_table[neigbours[i]-1][recipent_addr]< shortpath)
		{
			shortpath = node_table[neigbours[i]-1][recipent_addr];
			best_choice = neigbours[i];
//		usart_printstring("dir = inn, shortpath: %d best_choice %d, i: %d;    ", shortpath, best_choice, i);
		//retning ut fra sentralen
		}
		else if (node_table[recipent_addr-1][neigbours[i]] > 0 && node_table[recipent_addr][neigbours[i]]< shortpath){
			shortpath = node_table[recipent_addr][neigbours[i]];
			best_choice = neigbours[i];
//		usart_printstring("dir = ut, shortpath: %d best_choice %d, i: %d;    ", shortpath, best_choice, i);

		}
		else
			;
	}
	return best_choice;
}


int test_nodetable()
{}

void set_config(void)
{
    BIT_TRUE(PORTC, 7);
    delay_us(CONFIG_DELAY);
    BIT_FALSE(PORTC, 7);
}

char wait_for_config(){
	if(getchar != '<')
		return -1
	else return 1;
}

int change_addr(char addr)
{
	set_config()
	if(wait_for_config() == -1) return -1
		putchar('T');
	if(wait_for_config() == -1) return -1
		putchar(addr);
	if(wait_for_config() == -1) return -1
		putchar('X');

	return 1;
}
